@@ -16,6 +16,7 @@ types.
 .. autofunction:: find_file
 
 """
+
 from . import bruker_nmr
 from . import prospa
 from . import bruker_esr
@@ -24,6 +25,7 @@ from . import load_cary
 from .open_subpath import open_subpath
 from ..datadir import getDATADIR, rclone_search
 from ..datadir import pyspec_config, log_fname
+from .zenodo import zenodo_download
 from ..general_functions import strm
 from ..core import nddata_hdf5
 from numpy import r_
@@ -45,7 +47,13 @@ def _dirformat(file):
 
 
 # }}}
-def search_filename(searchstring, exp_type, print_result=True, unique=False):
+def search_filename(
+    searchstring,
+    exp_type,
+    print_result=True,
+    unique=False,
+    zenodo=None,
+):
     r"""Use regular expression `searchstring` to find a file inside the
     directory indicated by `exp_type`
     (For information on how to set up the file searching mechanism, see
@@ -59,6 +67,11 @@ def search_filename(searchstring, exp_type, print_result=True, unique=False):
         Teams or Google Drive, etc.)
     *   The user wants to seamlessly access the data on their laptop.
 
+    .. figure:: ../../doc_src/_static/presentation_images/image38.jpeg
+       :align: center
+
+       Bruker TopSpin interface showing a typical NMR directory.
+
     The ``.pyspecdata`` config file stores all the info about where the data
     lives + is stored locally.  You have basically two options:
 
@@ -101,6 +114,11 @@ def search_filename(searchstring, exp_type, print_result=True, unique=False):
         more info.
     unique : boolean (default False)
         If true, then throw an error unless only one file is found.
+    zenodo : str, optional
+        Deposition number on Zenodo.  If provided and the file is not found
+        locally, a file matching ``searchstring`` will be downloaded from this
+        deposition.  Rclone remotes are *not* searched when this option is
+        used.
     """
     # {{{ actually find the files
     directory = getDATADIR(exp_type=exp_type)
@@ -150,19 +168,24 @@ def search_filename(searchstring, exp_type, print_result=True, unique=False):
 
     files = look_inside(directory)
     logger.debug(strm("look_inside found the files", files))
-    if files is None or len(files) == 0:
-        rclone_search(
+    if (files is None or len(files) == 0) and zenodo is not None:
+        zenodo_download(zenodo, searchstring, exp_type=exp_type)
+    elif (files is None or len(files) == 0) and zenodo is None:
+        cmd = rclone_search(
             searchstring.replace(".*", "*")
             .replace("(", "{")
             .replace(")", "}")
-            .replace("|", ","),
+            .replace("|", ",")
+            .replace("\\.", "."),
             exp_type,
             directory,
         )
     files = look_inside(directory)
     if files is None or len(files) == 0:
         raise RuntimeError(
-            "even after rclone_search, I "
+            "even after rclone_search:\n"
+            + cmd
+            + "\nI "
             "can't find this file!\n"
             f"file search string: {searchstring}\n"
             f"exp type: {exp_type}\n"
@@ -173,13 +196,11 @@ def search_filename(searchstring, exp_type, print_result=True, unique=False):
                 map(
                     set,
                     list(
-                        zip(
-                            *[
-                                j.rsplit(".", 1)
-                                for j in files
-                                if len(j.rsplit(".", 1)) > 1
-                            ]
-                        )
+                        zip(*[
+                            j.rsplit(".", 1)
+                            for j in files
+                            if len(j.rsplit(".", 1)) > 1
+                        ])
                     ),
                 )
             )
@@ -202,7 +223,7 @@ def search_filename(searchstring, exp_type, print_result=True, unique=False):
             )
         elif len(retval) > 1:
             raise ValueError(
-                "found more than on file in",
+                "found more than one file in",
                 directory,
                 "matching",
                 searchstring,
@@ -222,6 +243,19 @@ def search_filename(searchstring, exp_type, print_result=True, unique=False):
     return retval
 
 
+def register_proc_lookup(newdict):
+    """this updates the dictionary at
+    pyspecdata.load_files.postproc_lookup
+    this is equivalent to passing newdict to the lookup
+    keyword argument of `find_file`.
+
+    Remember that you only have to pass the keyword argument or call this
+    function once per script, and the effects are persistent!
+    """
+    postproc_lookup.update(newdict)
+    return
+
+
 def find_file(
     searchstring,
     exp_type=None,
@@ -238,6 +272,7 @@ def find_file(
     indirect_dimlabels=None,
     lookup={},
     return_list=False,
+    zenodo=None,
     **kwargs,
 ):
     r"""Find the file  given by the regular expression `searchstring` inside
@@ -325,10 +360,17 @@ def find_file(
         available in pySpecData, but also, see the `lookup` argument,
         below).
 
+        Note that we call this "postprocessing" here because it follows
+        the data organization, *etc.*, performed by the rest of the file
+        in other contexts, however, we might call this "preprocessing"
+
         If `postproc` is a string,
         it looks up the string inside the `postproc_lookup`
         dictionary that's appropriate for the file type.
 
+        If `postproc` is `"none"`,
+        then explicitly do not apply any type of postprocessing.
+
         If `postproc` is None,
         it checks to see if the any of the loading functions that were
         called set the `postproc_type` property
@@ -357,6 +399,10 @@ def find_file(
                              :func:`~pyspecdata.load_files.load_indiv_file`
                              lookup : dictionary with str:function pairs
         types of postprocessing to add to the `postproc_lookup` dictionary
+    zenodo : str, optional
+        Deposition number on Zenodo.  When the requested file is not found
+        locally, a file matching ``searchstring`` will be downloaded from this
+        deposition instead of searching rclone remotes.
     """
     postproc_lookup.update(lookup)
     logger.debug(strm("find_file sees indirect_dimlabels", indirect_dimlabels))
@@ -373,7 +419,12 @@ def find_file(
             (searchstring, exp_type, print_result),
         )
     )
-    files = search_filename(searchstring, exp_type, print_result=print_result)
+    files = search_filename(
+        searchstring,
+        exp_type,
+        print_result=print_result,
+        zenodo=zenodo,
+    )
     # search_filename will raise an error if it can't find anything even after
     # checking remotely
     data = None
@@ -430,26 +481,36 @@ def find_file(
             logger.debug("got a postproc_type value of None")
             assert len(kwargs) == 0, (
                 "there must be no keyword arguments left, because you're done"
-                " postprocessing (you have %s)" % str(kwargs)
+                " postprocessing (you have %s)"
+                % str(kwargs)
             )
             return data
         else:
-            if postproc_type in list(postproc_lookup.keys()):
+            if postproc_type == "none":
+                logger.debug(
+                    "You specified you do not want any preprocessing"
+                    "applied so I am bypassing that to just give the raw"
+                    "data"
+                )
+            elif postproc_type in list(postproc_lookup.keys()):
                 data = postproc_lookup[postproc_type](data, **kwargs)
                 if "fl" in kwargs.keys():
                     kwargs.pop("fl")
                 logger.debug("this file was postprocessed successfully")
+            elif postproc_type == "stub":
+                pass
             else:
                 logger.debug(
                     "postprocessing not defined for file with postproc_type %s"
                     " --> it should be defined in the postproc_type dictionary"
-                    " in load_files.__init__.py" + postproc_type
+                    " in load_files.__init__.py"
+                    + postproc_type
                 )
-                if len(lookup) > 0:
+                if len(postproc_lookup.keys()) > 0:
                     raise ValueError(
-                        "You passed me a lookup argument, which implies that"
-                        " you think there is a defined postprocessing function"
-                        " for this data, which has postproc_type="
+                        "You have a postproc_lookup dictionary, which implies"
+                        " that you think there is a defined postprocessing"
+                        " function for this data, which has postproc_type="
                         + postproc_type
                         + ", but I'm not finding a postproc function for that"
                         " in the lookup dictionary that you provided me with"
@@ -540,13 +601,11 @@ def _check_signature(filename):
             thiskey in inistring for thiskey in list(file_signatures.keys())
         ):  # this will fail without overriding the numpy any( above
             retval = file_signatures[
-                next(
-                    (
-                        thiskey
-                        for thiskey in list(file_signatures.keys())
-                        if thiskey in inistring
-                    )
-                )
+                next((
+                    thiskey
+                    for thiskey in list(file_signatures.keys())
+                    if thiskey in inistring
+                ))
             ]
             logger.debug(strm("Found magic signature, returning", retval))
             return retval
@@ -640,13 +699,10 @@ def load_indiv_file(
                 os.path.normpath(filename).split(os.path.sep)[-1].split(".")[0]
             )
             assert all([j[0] == basename for j in list_of_files]), strm(
-                (
-                    "I expected that the zip file %s contains a directory"
-                    " called %s which contains your NMR data -- this appears"
-                    " not to be the case.  (Note that with future extensions,"
-                    " other formats will be possible.)"
-                )
-                % (filename, basename)
+                "I expected that the zip file %s contains a directory"
+                " called %s which contains your NMR data -- this appears"
+                " not to be the case.  (Note that with future extensions,"
+                " other formats will be possible.)" % (filename, basename)
             )
             file_reference = (zf, filename, basename)
         else:
@@ -676,7 +732,8 @@ def load_indiv_file(
         else:
             logger.debug(
                 "Identified a potential prospa file, because I didn't find ser"
-                " or acqus in " + strm(file_reference, expno_as_str)
+                " or acqus in "
+                + strm(file_reference, expno_as_str)
             )
             # the rest are for prospa
             # even though I've made steps towards supporting zipped prospa (by
@@ -738,7 +795,7 @@ def load_indiv_file(
                                     and "data" in f[j].keys()
                                 ):
                                     attrlist.append(j)
-                        logger.debug("return_list is ", return_list)
+                        logger.debug(strm("return_list is ", return_list))
                         if return_list:
                             print(
                                 "using return-list -- this should be"
@@ -747,7 +804,8 @@ def load_indiv_file(
                             return attrlist
                         raise ValueError(
                             "please select a node from the list and set to"
-                            " expno:\n\t" + "\n\t".join(attrlist)
+                            " expno:\n\t"
+                            + "\n\t".join(attrlist)
                         )
                     # assume this is a normal pySpecData HDF5 file
                     dirname, filename = os.path.split(filename)
@@ -908,18 +966,19 @@ def det_type(file, **kwargs):
 
 
 __all__ = [
-    "find_file",
-    "search_filename",
-    "load_indiv_file",
-    "format_listofexps",
-    "bruker_nmr",
-    "bruker_esr",
     "acert",
-    "prospa",
-    "load_t1_axis",
-    "bruker_load_t1_axis",
-    "prospa_t1_info",
-    "bruker_load_title",
     "bruker_dir",
+    "bruker_esr",
+    "bruker_load_t1_axis",
+    "bruker_load_title",
+    "bruker_nmr",
     "cw",
+    "find_file",
+    "format_listofexps",
+    "load_indiv_file",
+    "load_t1_axis",
+    "prospa",
+    "prospa_t1_info",
+    "register_proc_lookup",
+    "search_filename",
 ]
